/*
 * This is part of the doxml library, a parser for XML.  It should not
 *  require any functionality not defined in ANSI C's standard libraries.
 *
 * Copyright (C) 1999 John Stracke, http://www.thibault.org
 *
 * This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * $Log: getc.c,v $
 * Revision 1.6  1999/07/24 20:23:51  francis
 * Adding support for param entities in the DTD
 *
 * Revision 1.5  1999/07/24 13:43:46  francis
 * Adding support for namespaces in entity names.
 *
 * Revision 1.4  1999/06/01 01:51:14  francis
 * Adding functions to get/set/clear attributes.  Reorganzing the
 * to-do list.  Fixing a parser bug introduced I removed support for
 * attributes with no values at release 0.2.
 *
 * Revision 1.3  1999/05/21 23:05:42  francis
 * Finished renaming to doxml
 *
 * Revision 1.2  1999/05/21 22:59:48  francis
 * Partially renaming to doxml
 *
 * Revision 1.1.1.1  1999/05/21 22:20:36  francis
 * Initial checkin of libxml, to be renamed doxml
 *
 */

static const char* id="$Id: getc.c,v 1.6 1999/07/24 20:23:51 francis Exp $";

#include <doxml/priv.h>

#undef DEBUG /* this module gets way too verbose if DEBUG is turned on,
              *  so we switch it off.  If you want to track every character
              *  read, comment out this line.
              */

int
doxml_getc(doxml_context* context)
{
  if (!context)
    {
      fprintf(stderr,"doxml: doxml_getc(): context is NULL\n");
      return -1;
    }

  if ((context->ungetc_buff)>=0)
    {
      int res=context->ungetc_buff;
      context->ungetc_buff=-1;
      return res;
    }

  if (context->pushed)
    {
      int ch=doxml_getc(context->pushed);
      if (ch==EOF)
        {
          doxml_close(context->pushed);
          context->pushed=0;
        }
      else
        return ch;
    }

  if (!(context->reader.getc_method))
    {
      fprintf(stderr,"doxml: doxml_getc(): context has no getc method\n");
      return -1;
    }

  {
    int res=(*(context->reader.getc_method))(context);
#ifdef DEBUG
    fprintf(stderr,"doxml_getc(): ");
    if (res==EOF)
      fprintf(stderr,"<EOF>");
    else
      {
        if (res<0)
          fprintf(stderr,"<%d>",res);
        else
          fprintf(stderr,"%c",(char)res);
      }

    fprintf(stderr,"\n");
#endif
    switch (res)
      {
      case '\n':
        context->line_number++;
        break;

      case '%':
        if (context->doctype)
          {
            int res=doxml_doctype_handle_percent(context,context->doctype);
            if (res<0)
              {
                fprintf(stderr,
                        "doxml_getc(): %d: doxml_doctype_handle_percent() failed\n",
                        context->line_number);
                return -1;
              }
            if (res>0)
              {
                /* recurse to get the first character from the param value */
                return doxml_getc(context);
              }
            return '%';
          }
        break;

      default:
        break;
      }

    return res;
  }
}
