/*
 * This is part of the doxml library, a parser for XML.  It should not
 *  require any functionality not defined in ANSI C's standard libraries.
 *
 * Copyright (C) 1999 John Stracke, http://www.thibault.org
 *
 * This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * $Log: string.c,v $
 * Revision 1.5  1999/07/24 20:23:51  francis
 * Adding support for param entities in the DTD
 *
 * Revision 1.4  1999/05/21 23:10:29  francis
 * Reformatting, since renaming to doxml changed the widths of the
 * identifiers.
 *
 * Revision 1.3  1999/05/21 23:05:42  francis
 * Finished renaming to doxml
 *
 * Revision 1.2  1999/05/21 22:59:49  francis
 * Partially renaming to doxml
 *
 * Revision 1.1.1.1  1999/05/21 22:20:36  francis
 * Initial checkin of libxml, to be renamed doxml
 *
 */

static const char* id="$Id: string.c,v 1.5 1999/07/24 20:23:51 francis Exp $";

#include <doxml/priv.h>

typedef struct {
  const unsigned char* str;
  int offset;
  int free_str_after_parsing;
} string_closure;

static int string_getc_method(doxml_context* context)
{
  string_closure* closure;
  unsigned char ch;

  if (!context)
    return -1;

  closure=(string_closure*)(context->reader.closure);
  if (!closure)
    return -1;

  if (!(closure->str))
    return -1;

  ch=closure->str[closure->offset];
  if (!ch)
    return -1;

  closure->offset++;
  return (int)ch;
}

static void string_destructor_method(doxml_context* context)
{
  string_closure* closure;
  if (!context)
    return;

  closure=(string_closure*)(context->reader.closure);
  if (!closure)
    return;
  if (!closure->str)
    return;

  if (closure->free_str_after_parsing)
    free((void*)(closure->str)); /* cast away const for free() */
  free(closure);
  context->reader.closure=0;
}

doxml_context*
doxml_open_string(const char* str,
                  int free_str_after_parsing)
{
  string_closure* closure;
  doxml_context* context;

  if (!str)
    {
      fprintf(stderr,"doxml: doxml_open_string(): cannot parse null string\n");
      return 0;
    }

  context=doxml_construct_context(string_getc_method,
                                  string_destructor_method);
  if (!context)
    {
      fprintf(stderr,"doxml: doxml_open_string(): cannot construct context\n");
      return 0;
    }

  closure=(string_closure*)malloc(sizeof(string_closure));
  if (!closure)
    {
      fprintf(stderr,"doxml: doxml_open_string(): cannot malloc() closure\n");
      free(context);
      return 0;
    }

  closure->str=str;
  closure->offset=0;
  closure->free_str_after_parsing=free_str_after_parsing;
  context->reader.closure=closure;

  return context;
}
